// -*- C++ -*-
//
// This is the implementation of the non-inlined, non-templated member
// functions of the CascadeHandler class.
//

#include "CascadeHandler.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/EventRecord/Collision.h"
#include "ThePEG/EventRecord/SubProcess.h"
#include "ThePEG/EventRecord/Particle.h"
#include "ThePEG/Handlers/EventHandler.h"
#include "ThePEG/Handlers/Hint.h"
#include "ThePEG/Utilities/UtilityBase.h"
#include "Ariadne/DipoleCascade/DipoleState.h"

#ifdef ThePEG_TEMPLATES_IN_CC_FILE
// #include "CascadeHandler.tcc"
#endif

#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"

namespace Ariadne {

CascadeHandler::~CascadeHandler() {}

void CascadeHandler::cascade() {

  LorentzRotation rot;

  DipoleStatePtr state = new_ptr(DipoleState(this));

  tPVector final;
  tSubProPtr sub;

  Energy2 stot = 0.0*GeV2;

  // Decide whether we are cascadeing a whole sub-process or if we
  // only need to deal with final-state shower.
  if ( hint().tagged() && !tagged().empty() ) {
    final = tagged();
  } else {
    tCollPtr coll = eventHandler()->currentCollision();
    sub = coll->primarySubProcess();
    if ( !sub->incoming().first->coloured() &&
	 !sub->incoming().second->coloured() ) {
      final = tPVector(sub->outgoing().begin(), sub->outgoing().end());
    } else {
      tPPair inc = sub->incoming();
      if ( inc.first->coloured() ) inc.first = inc.first->parents()[0];
      if ( inc.second->coloured() ) inc.second = inc.second->parents()[0];
      rot = Utilities::getBoostToCM(inc);
      sub->transform(rot);
      stot = sub->shat();
      state->init(sub);
    }    
  }
  if ( !final.empty() ) {
    rot = Utilities::boostToCM(final.begin(), final.end());
    state->init(final);
    stot = Utilities::sumMomentum(final.begin(), final.end()).m2();
  }

  Energy2 pt2max = stot/4.0;
  Energy2 pt2min = sqr(pTCut());
  bool perf = false;

  while ( ( pt2max = state->select(pt2min, pt2max) ) > pt2min ) {
    if ( state->perform() ) perf = true;
  }

  rot.invert();
  if ( final.empty() ) {
    sub->transform(rot);
    if ( perf ) state->fill(sub, rot, newStep());
  } else {
    Utilities::transform(final.begin(), final.end(), rot);
    if ( perf ) state->fill(final, rot, newStep());
  }
}

void CascadeHandler::persistentOutput(PersistentOStream & os) const {
  os << theRunningCoupling << theAlpha0 << ounit(theLambdaQCD, GeV)
     << ounit(thePTCut, GeV) << theAlphaEM0 << ounit(thePTCutEM, GeV)
     << theNCol << theNFlav;
}

void CascadeHandler::persistentInput(PersistentIStream & is, int) {
  is >> theRunningCoupling >> theAlpha0 >> iunit(theLambdaQCD, GeV)
     >> iunit(thePTCut, GeV) >> theAlphaEM0 >> iunit(thePTCutEM, GeV)
     >> theNCol >> theNFlav;
}

ClassDescription<CascadeHandler> CascadeHandler::initCascadeHandler;
// Definition of the static class description member.

Energy CascadeHandler::minPTCut() const {
  return runningCoupling() == 1? lambdaQCD(): 0.0*GeV;
}

Energy CascadeHandler::maxLambdaQCD() const {
  return runningCoupling() == 1? pTCut(): Constants::MaxEnergy;
}

void CascadeHandler::Init() {

  static ClassDocumentation<CascadeHandler> documentation
    ("The Ariadne::CascadeHandler class administers the Ariadne dipole "
     "cascade.",
     "Parton cascades performed by Ariadne\\cite{Lav05} according to the "
     "Dipole Cascade Model\\cite{Gustafson:1986db,Gustafson:1988rq,"
     "Andersson:1989gp,Andersson:1990ki}.",
     "\\bibitem{Lav05}"
     "Nils Lavesson and Leif L\\\"onnblad, Preprint in preparation.\n"
     "\\bibitem{Gustafson:1986db}"
     "G\\\"osta Gustafson, Phys.~Lett.~{\\bf B175} (1986) 453.\n"
     "\\bibitem{Gustafson:1988rq}"
     "G\\\"osta Gustafson and Ulf Pettersson, "
     "Nucl.~Phys.~{\\bf B306} (1988) 746.\n"
     "\\bibitem{Andersson:1989gp}"
     "Bo Andersson, et al., Z.~Phys.~{\\bf C43} (1989) 625.\n"
     "\\bibitem{Andersson:1990ki}"
     "Bo Andersson, et al., Nucl.~Phys.~{\\bf B339} (1990) 393.");

  static Switch<CascadeHandler,int> interfaceRunningCoupling
    ("RunningCoupling",
     "Strategy for handling \\f$\\alpha_S\\f$.",
     &CascadeHandler::theRunningCoupling, 1, true, false);
  static SwitchOption interfaceRunningCouplingRunning
    (interfaceRunningCoupling,
     "Running",
     "Use a one loop running coupling with \\f$\\Lambda_{QCD}\\f$ given "
     "by <interface>LambdaQCD</interface>.",
     1);
  static SwitchOption interfaceRunningCouplingConstant
    (interfaceRunningCoupling,
     "Constant",
     "Use a constant coupling given by <interface>Alpha0</interface>.",
     0);
  static SwitchOption interfaceRunningCouplingExternal
    (interfaceRunningCoupling,
     "External",
     "Use whatever coupling is specified by the current StandardModelBase "
     "object.",
     -1);

  static Parameter<CascadeHandler,double> interfaceAlpha0
    ("Alpha0",
     "The constant \\f$\\alpha_S\\f$ to use if "
     "<interface>RunningCoupling</interface> is set to Constant.",
     &CascadeHandler::theAlpha0, 0.2, 0.0, 0,
     true, false, Interface::lowerlim);

  static Parameter<CascadeHandler,double> interfaceAlphaEM0
    ("AlphaEM0",
     "The constant \\f$\\alpha_{EM}\\f$ to use. If zero, use whatever "
    "is specified in the current StandardModelBase object.",
     &CascadeHandler::theAlphaEM0, 1.0/137.0, 0.0, 0,
     true, false, Interface::lowerlim);

  static Parameter<CascadeHandler,Energy> interfaceLambdaQCD
    ("LambdaQCD",
     "The \\f$\\Lambda_{QCD}\\f$ to use in the one loop running "
     "\\f$\\alpha_S\\f$ if <interface>RunningCoupling</interface> "
     "is set to Running.",
     &CascadeHandler::theLambdaQCD, GeV, 0.22*GeV, 0.0*GeV,
     Constants::MaxEnergy,
     true, false, Interface::limited,
     0, 0, 0, &CascadeHandler::maxLambdaQCD, 0);

  static Parameter<CascadeHandler,Energy> interfacePTCut
    ("PTCut",
     "The cutoff in invariant transverse momentum for QCD emissions.",
     &CascadeHandler::thePTCut, GeV, 0.6*GeV, 0.0*GeV, 0*GeV,
     true, false, Interface::lowerlim,
     0, 0, &CascadeHandler::minPTCut, 0, 0);

  static Parameter<CascadeHandler,Energy> interfacePTCutEM
    ("PTCutEM",
     "The cutoff in invariant transverse momentum for QED emissions.",
     &CascadeHandler::thePTCutEM, GeV, 0.6*GeV, 0.0*GeV, 0*GeV,
     true, false, Interface::lowerlim);

  static Parameter<CascadeHandler,int> interfaceDipoleColours
    ("DipoleColours",
     "The number of differently coloured dipoles possible. This should "
     "normally be 8, but may be varied to check the effects of colour "
     "reconnections.",
     &CascadeHandler::theNCol, 8, 3, 0,
     true, false, Interface::lowerlim);

  static Parameter<CascadeHandler,int> interfaceNFlav
    ("NFlav",
     "The number of possible flavours in a \f$g\to q\bar{q}\f$ splitting.",
     &CascadeHandler::theNFlav, 5, 0, 8,
     true, false, Interface::lowerlim);

}

}
