// -*- C++ -*-
#ifndef ARIADNE_CascadeHandler_H
#define ARIADNE_CascadeHandler_H
//
// This is the declaration of the CascadeHandler class.
//

#include "Ariadne/Config/Ariadne.h"
#include "ThePEG/Handlers/CascadeHandler.h"
#include "CascadeHandler.fh"

namespace Ariadne {

using namespace ThePEG;

/**
 * The CascadeHandler class inherits form the ThePEG::CascadeHandler
 * base class and implements the Dipole Cascade Model for partonic
 * cascades.
 *
 * @see \ref CascadeHandlerInterfaces "The interfaces"
 * defined for CascadeHandler.
 */
class CascadeHandler: public ThePEG::CascadeHandler {

public:

  /** @name Standard constructors and destructors. */
  //@{
  /**
   * The default constructor.
   */
  inline CascadeHandler();

  /**
   * The copy constructor.
   */
  inline CascadeHandler(const CascadeHandler &);

  /**
   * The destructor.
   */
  virtual ~CascadeHandler();
  //@}

public:

  /**
   * The main function to be overwritten by sub-classes. It is called
   * by handle() after storing some information which is then
   * available through simple access functions.
   */
  virtual void cascade();

public:

  /** @name Functions relating to the running coupling. */
  //@{
  /**
   * Strategy for \f$\alpha_S\f$. 0 means constant a constant
   * \f$\alpha_S\f$ with a value given by alpha0(). 1 means a leading
   * order running \f$\alpha_S\f$ with a \f$\Lambda_{QCD}\f$ given by
   * lambdaQCD(). -1 means take whatever is specified in the current
   * StandardModelBase object.
   */
  inline int runningCoupling() const;

  /**
   * The constant \f$\alpha_S\f$ to be used if runningCoupling() is 0.
   */
  inline double alpha0() const;

  /**
   * The \f$\Lambda_{QCD}\f$ to use in the one loop running
   * \f$\alpha_S\f$ if runningCoupling is 1
   */
  inline Energy lambdaQCD() const;

  /**
   * The constant \f$\alpha_{EM}\f$ to be used. If zero, use whatever
   * is specified in the current StandardModelBase object.
   */
  inline double alphaEM0() const;

  /**
   * The number of different colour indices available to dipoles.
   */
  inline int nCol() const;

  /**
   * The number of possible flavours in a \f$g\to q\bar{q}\f$ splitting.
   */
  inline int nFlav() const;
  //@}

public:

  /**
   * The cutoff in invariant transverse momentum for QCD emissions.
   */
  inline Energy pTCut() const;

  /**
   * The cutoff in invariant transverse momentum for QED emissions.
   */
  inline Energy pTCutEM() const;

public:

  /** @name Functions used by the persistent I/O system. */
  //@{
  /**
   * Function used to write out object persistently.
   * @param os the persistent output stream written to.
   */
  void persistentOutput(PersistentOStream & os) const;

  /**
   * Function used to read in object persistently.
   * @param is the persistent input stream read from.
   * @param version the version number of the object when written.
   */
  void persistentInput(PersistentIStream & is, int version);
  //@}

  /**
   * The standard Init function used to initialize the interfaces.
   * Called exactly once for each class by the class description system
   * before the main function starts or
   * when this class is dynamically loaded.
   */
  static void Init();

protected:

  /** @name Clone Methods. */
  //@{
  /**
   * Make a simple clone of this object.
   * @return a pointer to the new object.
   */
  inline virtual IBPtr clone() const;

  /** Make a clone of this object, possibly modifying the cloned object
   * to make it sane.
   * @return a pointer to the new object.
   */
  inline virtual IBPtr fullclone() const;
  //@}

protected:

  /** @name Standard Interfaced functions. */
  //@{
  /**
   * Check sanity of the object during the setup phase.
   */
  inline virtual void doupdate() throw(UpdateException);

  /**
   * Initialize this object after the setup phase before saving an
   * EventGenerator to disk.
   * @throws InitException if object could not be initialized properly.
   */
  inline virtual void doinit() throw(InitException);

  /**
   * Initialize this object. Called in the run phase just before
   * a run begins.
   */
  inline virtual void doinitrun();

  /**
   * Finalize this object. Called in the run phase just after a
   * run has ended. Used eg. to write out statistics.
   */
  inline virtual void dofinish();

  /**
   * Rebind pointer to other Interfaced objects. Called in the setup phase
   * after all objects used in an EventGenerator has been cloned so that
   * the pointers will refer to the cloned objects afterwards.
   * @param trans a TranslationMap relating the original objects to
   * their respective clones.
   * @throws RebindException if no cloned object was found for a given
   * pointer.
   */
  inline virtual void rebind(const TranslationMap & trans)
    throw(RebindException);

  /**
   * Return a vector of all pointers to Interfaced objects used in this
   * object.
   * @return a vector of pointers.
   */
  inline virtual IVector getReferences();
  //@}

private:

  /**
   * Function used by the interface.
   */
  Energy minPTCut() const;

  /**
   * Function used by the interface.
   */
  Energy maxLambdaQCD() const;

private:

  /**
   * Strategy for \f$\alpha_S\f$. 0 means constant a constant
   * \f$\alpha_S\f$ with a value given by alpha0(). 1 means a leading
   * order running \f$\alpha_S\f$ with a \f$\Lambda_{QCD}\f$ given by
   * lambdaQCD(). -1 means take whatever is specified in the current
   * StandardModelBase object.
   */
  int theRunningCoupling;

  /**
   * The constant \f$\alpha_S\f$ to be used if runningCoupling() is 0.
   */
  double theAlpha0;

  /**
   * The \f$\Lambda_{QCD}\f$ to use in the one loop running
   * \f$\alpha_S\f$ if runningCoupling is 1
   */
  Energy theLambdaQCD;

  /**
   * The cutoff in invariant transverse momentum for QCD emissions.
   */
  Energy thePTCut;

  /**
   * The constant \f$\alpha_{EM}\f$ to be used. If zero, use whatever
   * is specified in the current StandardModelBase object.
   */
  double theAlphaEM0;

  /**
   * The cutoff in invariant transverse momentum for QED emissions.
   */
  Energy thePTCutEM;

  /**
   * The number of different colour indices available to dipoles.
   */
  int theNCol;

  /**
   * The number of possible flavours in a \f$g\to q\bar{q}\f$ splitting.
   */
  int theNFlav;

private:

  /**
   * The static object used to initialize the description of this class.
   * Indicates that this is a concrete class with persistent data.
   */
  static ClassDescription<CascadeHandler> initCascadeHandler;

  /**
   * The assignment operator is private and must never be called.
   * In fact, it should not even be implemented.
   */
  CascadeHandler & operator=(const CascadeHandler &);

};

}

#include "ThePEG/Utilities/ClassTraits.h"

namespace ThePEG {

/** @cond TRAITSPECIALIZATIONS */

/** This template specialization informs ThePEG about the
 *  base classes of CascadeHandler. */
template <>
struct BaseClassTrait<Ariadne::CascadeHandler,1> {
  /** Typedef of the first base class of CascadeHandler. */
  typedef CascadeHandler NthBase;
};

/** This template specialization informs ThePEG about the name of
 *  the CascadeHandler class and the shared object where it is defined. */
template <>
struct ClassTraits<Ariadne::CascadeHandler>
  : public ClassTraitsBase<Ariadne::CascadeHandler> {
  /** Return a platform-independent class name */
  static string className() { return "Ariadne::CascadeHandler"; }
  /** Return the name of the shared library be loaded to get
   *  access to the CascadeHandler class and every other class it uses
   *  (except the base class). */
  static string library() { return "libArCascade.so"; }
};

/** @endcond */

}

#include "CascadeHandler.icc"
#ifndef ThePEG_TEMPLATES_IN_CC_FILE
// #include "CascadeHandler.tcc"
#endif

#endif /* ARIADNE_CascadeHandler_H */
