// -*- C++ -*-
//
// This is the implementation of the non-inlined, non-templated member
// functions of the Dipole class.
//

#include "Dipole.h"
#include "Parton.h"
#include "String.h"
#include "DipoleState.h"
#include "CascadeHandler.h"

#ifdef ThePEG_TEMPLATES_IN_CC_FILE
// #include "Dipole.tcc"
#endif

#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Repository/EventGenerator.h"
#include "ThePEG/PDT/EnumParticles.h"
#include "ThePEG/Utilities/SimplePhaseSpace.h"
#include "ThePEG/Utilities/UtilityBase.h"

using namespace Ariadne;

ClonePtr Dipole::clone() const {
  return new_ptr(*this);
}

void Dipole::init(tParPtr ip, tParPtr op) {
  iPart(ip);
  oPart(op);
  touch();
  lastPT2(-1.0*GeV2);
  setup(sdip()/4.0);
}

Energy2 Dipole::generate(Energy2 pt2min, Energy2 pt2max) {
  if ( !iPart()->touched() && !oPart()->touched() && !touched() ) {
    if ( lastPT2() > 0.0*GeV2 ) return lastPT2();
  }
  lastPT2(-1.0*GeV2);
  genG(pt2min, pt2max);
  genQ(max(lastPT2(), pt2min), pt2max);
  return lastPT2();
}

void Dipole::genQ(Energy2 pt2min, Energy2 pt2max) {
  Energy2 S = sdip();
  double C1 = iPart()->isG()?
    double(S/(4.0*Constants::pi*(S + prev()->sdip()))): 0.0;
  double C3 = oPart()->isG()?
    double(S/(4.0*Constants::pi*(S + next()->sdip()))): 0.0;
  if ( C1 + C3 <= 0.0 ) return;

  // We will pretent parton 3 is the gluon even if it isn't. Hence
  // parton 1 may have a mass or not.
  double y1 = 0.0;
  if ( !iPart()->isG() ) y1 = max(iPart()->momentum().m2()/S, 0.0);
  if ( !oPart()->isG() ) y1 = max(oPart()->momentum().m2()/S, 0.0);
  double sy1 = sqrt(y1);
  Energy W = sqrt(S);
  pt2max = min(pt2max, S/4.0);
  if ( pt2max <= pt2min ) return;

  for ( int ifl = 1; ifl <= handler()->nFlav(); ++ifl ) {

    tcPDPtr q = handler()->generator()->getParticleData(ifl);
    if ( !q ) continue;
    Energy mq = q->generateMass();
    double syq = mq/W;
    double yq = sqr(syq);

    Energy2 pt2m = pt2max;
    if ( pt2m <= pt2min ) continue;
    double yint = 2.0*acosh(sqrt(S/pt2min)/2.0);

    while ( true ) {
      pt2m = rndsud(2.0*(C1 + C3)*yint, pt2m, pt2min);
      if ( pt2m <= pt2min ) break;
      double ymax = acosh(sqrt(S/pt2m)/2.0);
      double y = 2.0*ymax*handler()->rnd() - ymax;
      double xx1 = 1.0 - sqrt(pt2m/S)*exp(-y) + y1 - 4.0*yq;
      double xx3 = 1.0 - sqrt(pt2m/S)*exp( y) + yq - sqr(sy1 + syq);

      if ( !check(xx1, xx3, y1, yq, yq) ) continue;

      double weight = 2.0*ymax/yint;

      // *** ATTENTION *** Add other weights here.

      if ( weight < handler()->rnd() ) continue;
      
      lastPT2(pt2m);
      qmass = mq;
      x1 = xx1;
      x3 = xx3;
      flav = handler()->rndbool(C1, C3)? -ifl: ifl;
      pt2min = pt2m;
      break;
    }
  }

}

void Dipole::genG(Energy2 pt2min, Energy2 pt2max) {
  double C = (iPart()->isG() || oPart()->isG()? 3.0/4.0: 2.0/3.0)/
    Constants::pi;
  int n1 =  iPart()->isG()? 3: 2;
  int n3 =  oPart()->isG()? 3: 2;
  Energy2 S = sdip();
  double y1 = max(iPart()->momentum().m2()/S, 0.0);
  double y3 = max(oPart()->momentum().m2()/S, 0.0);
  pt2max = min(pt2max, S/4.0);
  if ( pt2max <= pt2min ) return;
  double yint = 2.0*acosh(sqrt(S/pt2min)/2.0);

  while (true) {
    pt2max = rndsud(2.0*C*yint, pt2max, pt2min);
    if ( pt2max <= pt2min ) return;
    double ymax = acosh(sqrt(S/pt2max)/2.0);
    double y = 2.0*ymax*handler()->rnd() - ymax;
    double xx1 = 1.0 - sqrt(pt2max/S)*exp(-y) + y1 - y3;
    double xx3 = 1.0 - sqrt(pt2max/S)*exp( y) + y3 - y1;

    if ( !check(xx1, xx3, y1, 0.0, y3) ) continue;

    double weight = (pow(xx1, n1) + pow(xx3, n3))/2.0;
    weight *= 2.0*ymax/yint;

    //Special handling of heavy quarks, the 'dead-cone' effect.
    double xm = exp(-2.0*y);
    weight *= 1.0 - (y1*xm + y3/xm)/(xx1 + xx3 - 1.0);

    if ( weight < handler()->rnd() ) continue;

    lastPT2(pt2max);
    flav = ParticleID::g;
    x1 = xx1;
    x3 = xx3;
    return;
  }
}

bool Dipole::perform() {
  if ( !flav ) return false;
  else if ( flav == ParticleID::g ) return performG();
  else return performQ();
}

bool Dipole::performQ() {
  tParPtr gluon = flav > 0? oPart(): iPart();
  tParPtr other = flav > 0? iPart(): oPart();
  LorentzMomentum p1(0.0*GeV, 0.0*GeV, 0.0*GeV,
		     sqrt(max(other->momentum().m2(), 0.0*GeV2)));
  LorentzMomentum p2(0.0*GeV, 0.0*GeV, 0.0*GeV, qmass);
  LorentzMomentum p3(0.0*GeV, 0.0*GeV, 0.0*GeV, qmass);

  try {
    SimplePhaseSpace::CMS(p1, p2, p3, sdip(), x1, x3, 0.0, 0.0, 0.0);
  }
  catch ( ImpossibleKinematics ) {
    return false;
  }

  LorentzRotation R;
  R.rotateZ(handler()->rnd(2.0*Constants::pi));
  R.transform(Utilities::getBoostFromCM(make_pair(other->momentum(),
						  gluon->momentum())));
  p1.transform(R);
  p2.transform(R);
  p3.transform(R);

  other->momentum() = p1;
  return splitGluon(p2, p3);

}

bool Dipole::performG() {
  LorentzMomentum p1(0.0*GeV, 0.0*GeV, 0.0*GeV,
		     sqrt(max(iPart()->momentum().m2(), 0.0*GeV2)));
  LorentzMomentum p2;
  LorentzMomentum p3(0.0*GeV, 0.0*GeV, 0.0*GeV,
		     sqrt(max(oPart()->momentum().m2(), 0.0*GeV2)));
  try {
    SimplePhaseSpace::CMS(p1, p2, p3, sdip(), x1, x3, 0.0, 0.0, 0.0);
  }
  catch ( ImpossibleKinematics ) {
    return false;
  }
  double Psi = Constants::pi - p3.theta();
  double beta = 0.0;
  if ( iPart()->isG() && oPart()->isG() )
    beta = Psi*sqr(x3)/(sqr(x1) + sqr(x3)); // minimize pt
  else if ( oPart()->isG() )
    beta = Psi;
  else if ( !iPart()->isG() && sqr(x3) > handler()->rnd()*(sqr(x1) + sqr(x3)) )
    beta = Psi;

  LorentzRotation R;
  R.rotateY(-beta);
  R.rotateZ(handler()->rnd(2.0*Constants::pi));
  R.transform(Utilities::getBoostFromCM(make_pair(iPart()->momentum(),
						  oPart()->momentum())));
  p1.transform(R);
  p2.transform(R);
  p3.transform(R);

  iPart()->momentum() = p1;
  oPart()->momentum() = p3;
  ParPtr g = emitGluon(p2);
  if ( !g ) return false;

  return true;
}

bool Dipole::splitGluon(const LorentzMomentum & p2,
			const LorentzMomentum & p3) {
  ParPtr q = state()->create<Parton>();
  q->data(handler()->generator()->getParticleData(abs(flav)));
  ParPtr qbar = state()->create<Parton>();
  qbar->data(handler()->generator()->getParticleData(-abs(flav)));
  touch();
  if ( flav > 0 ) {
    q->parents(make_pair(oPart(), iPart()));
    qbar->parents(make_pair(oPart(), iPart()));
    q->momentum() = p2;
    qbar->momentum() = p3;
    next()->touch();
    iPart()->touch();
    oPart()->string()->split(oPart(), q, qbar);
  } else {
    q->parents(make_pair(iPart(), oPart()));
    qbar->parents(make_pair(iPart(), oPart()));
    qbar->momentum() = p2;
    q->momentum() = p3;
    prev()->touch();
    oPart()->touch();
    iPart()->string()->split(iPart(), q, qbar);
  }


  return true;
}

ParPtr Dipole::emitGluon(const LorentzMomentum & pg) {
  ParPtr g = state()->create<Parton>();
  g->data(handler()->generator()->getParticleData(ParticleID::g));
  DipPtr d = state()->create<Dipole>();
  if ( !g || !d ) return g;
  g->momentum() = pg;
  iPart()->touch();
  oPart()->touch();
  touch();
  g->parents(make_pair(iPart(), oPart()));
  oPart()->iDip(d);
  d->init(g, oPart());
  g->oDip(d);
  g->iDip(this);
  oPart(g);

  g->string(iPart()->string());
  d->generateColourIndex();

  return g;
}

tDipPtr Dipole::prev() const {
  return iPart()? iPart()->iDip(): tDipPtr();
}

tDipPtr Dipole::next() const {
  return oPart()? oPart()->oDip(): tDipPtr();
}

Energy2 Dipole::sdip() const {
  return (iPart()->momentum() + oPart()->momentum()).m2();
}

void Dipole::generateColourIndex() {
  int ncol = handler()->nCol();
  if ( ncol < 3 ) return;
  int idx1 = prev()? prev()->colourIndex(): 0;
  int idx3 = next()? next()->colourIndex(): 0;
  int strndx = 0;
  if ( idx1 ) strndx = idx1/div;
  else if ( idx3 ) strndx = idx3/div;
  do {
    colourIndex(handler()->irnd(ncol) + 1 + strndx*div);
  } while ( colourIndex() == idx1 || colourIndex() == idx3 );
}

void Dipole::fillReferences(CloneSet & cset) {
  CascadeBase::fillReferences(cset);
  cset.insert(oPart());
  cset.insert(iPart());
}

void Dipole::rebind(const TranslationMap & trans) {
  CascadeBase::rebind(trans);
  theOPart = trans.translate(oPart());
  theIPart = trans.translate(iPart());
}

void Dipole::persistentOutput(PersistentOStream & os) const {
  os << theIPart << theOPart << theColourIndex << flav << x1 << x3
     << ounit(qmass, GeV);
}

void Dipole::persistentInput(PersistentIStream & is, int) {
  is >> theIPart >> theOPart >> theColourIndex >> flav >> x1 >> x3
     >> iunit(qmass, GeV);
}

ClassDescription<Dipole> Dipole::initDipole;
// Definition of the static class description member.

void Dipole::Init() {}

