// -*- C++ -*-
//
// This is the implementation of the non-inlined, non-templated member
// functions of the DipoleState class.
//

#include "DipoleState.h"
#include "Parton.h"
#include "Dipole.h"
#include "String.h"
#include "ThePEG/EventRecord/SubProcess.h"
#include "ThePEG/EventRecord/Particle.h"
#include "ThePEG/EventRecord/ColourSinglet.h"
#include "ThePEG/EventRecord/ColourLine.h"
#include "ThePEG/EventRecord/Step.h"
#include "ThePEG/Utilities/ObjectIndexer.h"

#ifdef ThePEG_TEMPLATES_IN_CC_FILE
// #include "DipoleState.tcc"
#endif

#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"

using namespace Ariadne;

ClonePtr DipoleState::clone() const {
  return new_ptr(*this);
}

bool DipoleState::init(tSubProPtr sub) {
  throw RemnantException()
    << "The Ariadne::CascadeHandler '" << handler()->name()
    << "' was askec to cascade partons from incoming hadrons. "
    << "This is not yet implemented." << Exception::runerror;
  return false;
}

bool DipoleState::init(const tPVector & out) {
  // First extract all singlets.
  vector<ColourSinglet> singlets =
    ColourSinglet::getSinglets(out.begin(), out.end());
  if ( singlets.empty() ) return false;

  // Now extract all strings, partons and connect them with dipoles.
  for ( int i = 0, N = singlets.size(); i < N; ++i ) {
    ColourSinglet & sing = singlets[i];
    if ( sing.nPieces() > 1 ) throw JunctionException()
      << "The Ariadne::CascadeHandler '" << handler()->name()
      << "' cannot handle junction strings." << Exception::runerror;
    bool gluonring = false;
    bool reverse = false;
    if ( sing.partons().size() < 2 ) throw StringException()
      << "The Ariadne::CascadeHandler '" << handler()->name()
      << "' found a string with less than two partons." << Exception::runerror;
    if ( sing.partons()[0]->hasAntiColour() ) {
      if ( sing.partons()[0]->hasColour() ) gluonring = true;
      else reverse = true;
    }
    StrPtr str = create<String>();
    vector<tParPtr> parts(sing.partons().size());
    for ( int j = 0, M = sing.partons().size(); j < M; ++j ) {
      parts[j] = create<Parton>();
      parts[j]->parent(sing.partons()[j]);
      parts[j]->string(str);
    }
    if ( reverse ) std::reverse(parts.begin(), parts.end());
    str->endpoints(*parts.begin(), parts.back());
    strings.insert(str);
    
    for ( int j = 1, M = sing.partons().size(); j < M; ++j ) {
      DipPtr dip = create<Dipole>();
      dip->init(parts[j], parts[j - 1]);
      parts[j - 1]->iDip(dip);
      parts[j]->oDip(dip);
      dip->generateColourIndex();
    }
    if ( gluonring ) {
      DipPtr dip = create<Dipole>();
      dip->init(parts[0], parts.back());
      parts.back()->iDip(dip);
      parts[0]->oDip(dip);
      dip->generateColourIndex();
    }
  }
  
  return true;
}

Energy2 DipoleState::select(Energy2 pt2min, Energy2 pt2max) {
  theSelected = tEmiPtr();
  Energy2 pt2 = 0.0*GeV2;
  Energy2 pt2sel = pt2min;
  for ( EmitterSet::iterator it = emitters.begin();
	it != emitters.end(); ++it )
    if ( (pt2 = (**it).generate(pt2min, pt2max)) > pt2sel ) {
      theSelected = *it;
      pt2sel = pt2;
    }
  for ( BaseSet::iterator it = objects.begin(); it != objects.end(); ++it)
    (**it).untouch();
  return pt2sel;
}

bool DipoleState::perform() {
  if ( !selected() ) return false;
  if ( !selected()->perform() ) {
    selected()->touch();
    return false;
  }
  return true;
}

void DipoleState::
fill(const tPVector & initial, const LorentzRotation & rot, tStepPtr step) {

  ObjectIndexer<tParPtr,Particle> passoc;
  PVector final;
  for ( PartonSet::iterator it = partons.begin(); it != partons.end(); ++it ) {
    final.push_back((**it).data().produceParticle(rot*(**it).momentum()));
    passoc(*it, final.back());
  }

  const ObjectIndexer<tParPtr,Particle> & cassoc = passoc;
  for ( PartonSet::iterator it = partons.begin(); it != partons.end(); ++it )
    if ( (**it).oDip() )
      ColourLine::create(cassoc((**it).next()), cassoc(*it));

  step->addDecayProduct(initial.begin(), initial.end(),
			final.begin(), final.end());
}


void DipoleState::
fill(tSubProPtr sub, const LorentzRotation & rot, tStepPtr step) {}


void DipoleState::rebind(const TranslationMap & trans) {
  BaseSet old;
  old.swap(objects);
  objects.clear();
  trans.translate(inserter(objects), old.begin(), old.end());
  PartonSet pold;
  pold.swap(partons);
  partons.clear();
  trans.translate(inserter(partons), pold.begin(), pold.end());
  EmitterSet eold;
  eold.swap(emitters);
  emitters.clear();
  trans.translate(inserter(emitters), eold.begin(), eold.end());
  StringSet sold;
  sold.swap(strings);
  strings.clear();
  trans.translate(inserter(strings), sold.begin(), sold.end());
  theSelected = trans.translate(theSelected);
}

DipoleStatePtr DipoleState::fullclone() const {
  DipoleStatePtr copy = dynamic_ptr_cast<DipoleStatePtr>(clone());
  TranslationMap trans;
  vector<ClonePtr> copies;
  
  copies.push_back(trans[tcDipoleStatePtr(this)] = copy);
  for ( BaseSet::const_iterator it = objects.begin();
	it != objects.end(); ++it )
    copies.push_back(trans[*it] = (**it).clone());

  for ( int i = 0, N = copies.size(); i < N; ++i ) copies[i]->rebind(trans);

  return copy;
}

void DipoleState::remove(tParPtr p) {
  partons.erase(p);
}

void DipoleState::persistentOutput(PersistentOStream & os) const {
  os << partons << emitters << strings << objects << theSelected;
}

void DipoleState::persistentInput(PersistentIStream & is, int) {
  is >> partons >> emitters >> strings >> objects >> theSelected;
}

ClassDescription<DipoleState> DipoleState::initDipoleState;
// Definition of the static class description member.

void DipoleState::Init() {}

