// -*- C++ -*-
#ifndef ARIADNE_DipoleState_H
#define ARIADNE_DipoleState_H
//
// This is the declaration of the DipoleState class.
//

#include "CascadeBase.h"
#include "DipoleState.fh"
#include "Emitter.fh"
#include "Parton.fh"
#include "String.fh"

namespace Ariadne {

/**
 * DipoleState contains all Ariadne::Parton, Ariadne::Dipole and
 * Ariadne::String objects needed to describe a partonic state within
 * the Dipole Cascade Model.
 */
class DipoleState: public CascadeBase {

public:

  /**
   * A set of CascadeBase pointers.
   */
  typedef set<CascadeBasePtr> BaseSet;

  /**
   * A set of Parton pointers.
   */
  typedef set<tParPtr> PartonSet;

  /**
   * A set of Emitter pointers.
   */
  typedef set<tEmiPtr> EmitterSet;

  /**
   * A set of String pointers.
   */
  typedef set<tStrPtr> StringSet;


public:

  /** @name Standard constructors and destructors. */
  //@{
  /**
   * The default constructor.
   */
  inline DipoleState(tHandlerPtr hdl = tHandlerPtr());

  /**
   * The copy constructor.
   */
  inline DipoleState(const DipoleState &);

  /**
   * The destructor.
   */
  inline virtual ~DipoleState();
  //@}

public:

  /** @name The main functions controlling the cascade. */
  //@{
  /**
   * Initialize this state. Using information about the hard
   * sub-process, initialize the DipoleState, preparing for the
   * cascade.
   *
   * @param sub the SubProcess
   *
   * @return false if no cascade is possible.
   */
  bool init(tSubProPtr sub);

  /**
   * Init simple final-state cascade. Using the outgoing partons in \a
   * out to initialize a final-state cascade.
   *
   * @return false if no cascade was possible.
   */
  bool init(const tPVector & out);

  /**
   * Select an Emitter. Go through the list of Emitter objects and
   * tell them to calculate the invariant transverse momentum of the
   * next emission and select the Emitter with largest transverse
   * momentum. Subsequent calls to selected() will return the selected
   * Emitter.
   *
   * @param pt2min the minimum allowed transverse momentum.
   *
   * @param pt2max the maximum allowed transverse momentum.
   *
   * @return the generated invariant squared transverse momentum. If
   * ptmin is returned, no emission was generated and subsequent calls
   * to selected() will return null.
   */
  Energy2 select(Energy2 pt2min, Energy2 pt2max);

  /**
   * Perform an emission.
   *
   * @return false if no emission was possible.
   */
  bool perform();

  /**
   * Output the result of a simple final-state cascade. The new
   * partons are created, boosted and inserted in the Step provided.
   *
   * @param inital the initial list of partons before the cascade.
   *
   * @param rot the LorentzRotation by which the final partons will be
   * bosted.
   *
   * @param step the step in which to insert the produced partons.
   */
  void fill(const tPVector & initial, const LorentzRotation & rot, tStepPtr step);

  /**
   * Output the result of a cascade starting from a full sub
   * process. The resulting partons are created, boosted and inserted
   * in the Step provided.
   *
   * @param sub the initial SubProcess object.
   *
   * @param rot the LorentzRotation by which the final partons will be
   * bosted.
   *
   * @param step the step in which to insert the produced partons.
   */
  void fill(tSubProPtr sub, const LorentzRotation & rot, tStepPtr step);
  //@}

  /** @name Simple access functions. */
  //@{
  /**
   * The Emitter selected to perform the next emission.
   */
  inline tEmiPtr selected() const;  
  //@}

  /** @name Functions relating to the book keeping of included objects. */
  //@{
  /**
   * Create a new object to be included in this state.
   */
  template <typename Class>
  inline typename Ptr<Class>::pointer create();

  /**
   * Remove the given parton. Not that it is only removed from the
   * list of partons. It is still remembered as an object belonging to
   * this state.
   */
  void remove(tParPtr p);

  /**
   * Clone this state. Also cloning all included objects and fixing up
   * their inter-dependence.
   */
  DipoleStatePtr fullclone() const;
  //@}

protected:

  /** @name The virtual functions to be overridden in sub-classes. */
  //@{
  /**
   * Return a simple clone of this object. Should be implemented as
   * <code>return new_ptr(*this);</code> by a derived class.
   */
  virtual ClonePtr clone() const;

  /**
   * Fill the provided set with all pointers to CloneBase objects used
   * in this object.
   */
  inline virtual void fillReferences(CloneSet &);

  /**
   * Rebind pointer to other CloneBase objects. Called after a number
   * of interconnected CloneBase objects have been cloned, so that
   * the cloned objects will refer to the cloned copies afterwards.
   *
   * @param trans a TranslationMap relating the original objects to
   * their respective clones.
   */
  virtual void rebind(const TranslationMap & trans);
  //@}

public:

  /** @name Functions used by the persistent I/O system. */
  //@{
  /**
   * Function used to write out object persistently.
   * @param os the persistent output stream written to.
   */
  void persistentOutput(PersistentOStream & os) const;

  /**
   * Function used to read in object persistently.
   * @param is the persistent input stream read from.
   * @param version the version number of the object when written.
   */
  void persistentInput(PersistentIStream & is, int version);
  //@}

  /**
   * The standard Init function used to initialize the interfaces.
   * Called exactly once for each class by the class description system
   * before the main function starts or
   * when this class is dynamically loaded.
   */
  static void Init();

public:

  /** @cond EXCEPTIONCLASSES */
  /**
   * Exception class used if a junction string was found - we don't
   * know how to cascade them yet.
   */
  struct JunctionException: public Exception {};

  /**
   * Exception class used if a hadron remnants were found - we can't
   * cascade them yet.
   */
  struct RemnantException: public Exception {};

  /**
   * Exception class used if a string with less than two partons was
   * found.
   */
  struct StringException: public Exception {};
  /** @endcond */

protected:

  /**
   * These are all the final-state partons included in this state.
   */
  mutable PartonSet partons;

  /**
   * These are all the emitters included in this state.
   */
  mutable EmitterSet emitters;

  /**
   * These are all the strings included in this state.
   */
  mutable StringSet strings;

private:

  /**
   * A set containing all objects included in this state.
   */
  mutable BaseSet objects;

  /**
   * The Emitter selected to perform the next emission.
   */
  tEmiPtr theSelected;

private:

  /**
   * The static object used to initialize the description of this class.
   * Indicates that this is a concrete class with persistent data.
   */
  static ClassDescription<DipoleState> initDipoleState;

  /**
   * The assignment operator is private and must never be called.
   * In fact, it should not even be implemented.
   */
  DipoleState & operator=(const DipoleState &);

};

}

#include "ThePEG/Utilities/ClassTraits.h"

namespace ThePEG {

/** @cond TRAITSPECIALIZATIONS */

/** This template specialization informs ThePEG about the
 *  base classes of DipoleState. */
template <>
struct BaseClassTrait<Ariadne::DipoleState,1> {
  /** Typedef of the first base class of DipoleState. */
  typedef Ariadne::CascadeBase NthBase;
};

/** This template specialization informs ThePEG about the name of
 *  the DipoleState class and the shared object where it is defined. */
template <>
struct ClassTraits<Ariadne::DipoleState>
  : public ClassTraitsBase<Ariadne::DipoleState> {
  /** Return a platform-independent class name */
  static string className() { return "Ariadne::DipoleState"; }
  /** Return the name of the shared library be loaded to get
   *  access to the DipoleState class and every other class it uses
   *  (except the base class). */
  static string library() { return "libArCascade.so"; }
};

/** @endcond */

}

#include "DipoleState.icc"
#ifndef ThePEG_TEMPLATES_IN_CC_FILE
// #include "DipoleState.tcc"
#endif

#endif /* ARIADNE_DipoleState_H */
