// -*- C++ -*-
//
// This is the implementation of the non-inlined, non-templated member
// functions of the Emitter class.
//

#include "Emitter.h"
#include "DipoleState.h"

#ifdef ThePEG_TEMPLATES_IN_CC_FILE
// #include "Emitter.tcc"
#endif

#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/StandardModel/StandardModelBase.h"
#include "ThePEG/Repository/EventGenerator.h"

using namespace Ariadne;

Energy2 Emitter::rndsud(double C, Energy2 pt2max, Energy2 pt2min) const {
  double CN = 1.0/(C*alpha0());
  bool reweight = handler()->runningCoupling() < 0;
  while ( true ) {
    double R = handler()->rnd();
    if ( lambdaQCD2() <= 0.0*GeV2 )
      return CN*log(R) < log(pt2min/pt2max)? -1.0*GeV2: pt2max*pow(R, CN);
    if ( CN*log(R) < log(log(pt2min/lambdaQCD2())/log(pt2max/lambdaQCD2())) )
      return -1.0*GeV2;
    pt2max = lambdaQCD2()*pow(pt2max/lambdaQCD2(),pow(R, CN));
    if ( !reweight ) return pt2max;
    double weight = handler()->standardModel()->alphaS(pt2max)*
      log(pt2max/lambdaQCD2())/alpha0();
    if ( weight > 1.0 ) handler()->generator()->logWarning(
      WeightException() << "The Ariadne::CascadeHandler '" << handler()->name()
      << "' failed to overestimate the alpha_S specified by the StandardModel "
      << "object. If this hapens too often you should contact the authors."
      << Exception::warning);			   
    if ( handler()->rnd() < weight ) return pt2max;
  }
}

Energy2 Emitter::rndsudEM(double C, Energy2 pt2max, Energy2 pt2min) const {
  double aEM = handler()->alphaEM0() > 0.0? handler()->alphaEM0():
    handler()->standardModel()->alphaEM(pt2max);
  double CN = 1.0/(C*aEM);
  while ( true ) {
    double R = handler()->rnd();
    if ( CN*log(R) < log(pt2min/pt2max) ) return -1.0*GeV2;
    pt2max = pt2max*pow(R, CN);
    double weight = handler()->standardModel()->alphaEM(pt2max)/aEM;
    if ( weight > 1.0 ) handler()->generator()->logWarning(
      WeightException() << "The Ariadne::CascadeHandler '" << handler()->name()
      << "' failed to overestimate the alpha_EM specified by the StandardModel "
      << "object. If this hapens too often you should contact the authors."
      << Exception::warning);			   
    if ( handler()->rnd() < weight ) return pt2max;
  }
}

void Emitter::setup(Energy2 pt2max) {
  const StandardModelBase & SM = *handler()->standardModel();
  using namespace ThePEG::Constants;
  switch ( handler()->runningCoupling() ) {
  case 0:
    theAlpha0 = handler()->alpha0();
    return;
  case 1:
    theLambdaQCD2 = sqr(handler()->lambdaQCD());
    theAlpha0 = 12.0*Constants::pi/(33.0 - 2.0*SM.Nf(pt2max));
    return;
  }

  Energy2 s1 = sqr(handler()->pTCut());
  double a1 = SM.alphaS(s1);
  Energy2 s2 = pt2max;
  double a2 = SM.alphaS(s2);
  if ( a1 == a2 ) {
    theLambdaQCD2 = -1.0*GeV2;
    theAlpha0 = a1;
  } else {
    theLambdaQCD2 = exp(log(s2/s1)/(1.0 - a1/a2))*s1;
    theAlpha0 = a1*log(s1/lambdaQCD2());
  }
}

void Emitter::persistentOutput(PersistentOStream & os) const {
  os << ounit(theLastPT2, GeV2) << theAlpha0 << ounit(theLambdaQCD2, GeV2);
}

void Emitter::persistentInput(PersistentIStream & is, int) {
  is >> iunit(theLastPT2, GeV2) >> theAlpha0 >> iunit(theLambdaQCD2, GeV2);
}

AbstractClassDescription<Emitter> Emitter::initEmitter;
// Definition of the static class description member.

void Emitter::Init() {}

