// -*- C++ -*-
#ifndef ARIADNE_Emitter_H
#define ARIADNE_Emitter_H
//
// This is the declaration of the Emitter class.
//

#include "CascadeBase.h"
#include "Emitter.fh"

namespace Ariadne {

using namespace ThePEG;

/**
 * Emitter is the base class of all classes able to perform parton
 * emissions according to the Dipole Cascade Model and its extensions.
 */
class Emitter: public CascadeBase {

public:

  /**
   * The DipoleState is a friend.
   */
  friend class DipoleState;

public:

  /** @name Standard constructors and destructors. */
  //@{
  /**
   * The default constructor.
   */
  inline Emitter();

  /**
   * The copy constructor.
   */
  inline Emitter(const Emitter &);

  /**
   * The destructor.
   */
  inline virtual ~Emitter();
  //@}

public:

  /** @name The main virtual functions to be overridden by sub-classes. */
  //@{
  /**
   * Generate the variables needed to perform an emission. The
   * variables should be saved so that the emission can be performed
   * by a subsequent call to perform().
   *
   * @param pt2min the minimum transverse momentum squared allowed.
   *
   * @param pt2max the minimum transverse momentum squared allowed.
   *
   * @return the transverse momentum squared of the generated
   * emission. If 0 is returned, no emission was generated.
   */
  virtual Energy2 generate(Energy2 pt2min, Energy2 pt2max) = 0;

  /**
   * Perform an emission. Use the information previously generated by
   * generate(Energy2,Energy2) to perform the corresponding emission.
   *
   * @return false if the emission failed for whatever reason.
   */
  virtual bool perform() = 0;

  /**
   * Setup internal constants and prepare to be able to generate. \a
   * pt2max is the maximum possible squared transverse momentum
   * possible for this emitter.
   */
  virtual void setup(Energy2 pt2max);
  //@}

  /**
   * Generate an squared invariant transverse momentum. The
   * distribution used is \f$C\alpha_S\frac{dp_\perp^2}{p_\perp^2}\f$
   * multiplied with the corresponding Sudakov formfactor. The maximum
   * \f$p_\perp^2\f$ is \a pt2max and \a C is the constant. This
   * automatically takes care of whether we are using a running
   * \f$\alpha_S\f$ or whatever.
   */
  Energy2 rndsud(double C, Energy2 pt2max, Energy2 pt2min) const;

  /**
   * Generate an squared invariant transverse momentum. The
   * distribution used is \f$C\alpha_{EM}\frac{dp_\perp^2}{p_\perp^2}\f$
   * multiplied with the corresponding Sudakov formfactor. The maximum
   * \f$p_\perp^2\f$ is \a pt2max and \a C is the constant. This
   * automatically takes care of whether we are using a running
   * \f$\alpha_{EM}\f$ or whatever.
   */
  Energy2 rndsudEM(double C, Energy2 pt2max, Energy2 pt2min) const;

public:

  /** @name Simple access functions. */
  //@{
  /**
   * The scale of the last generated emission.
   */
  inline Energy2 lastPT2() const;

  /**
   * The constant \f$\alpha_S\f$ or the constant to be divided by the
   * log of the scale.
   */
  inline double alpha0() const;

  /**
   * The \f$\lambda{QCD}^2\f$ used in the running \f$\alpha_S\f$. If
   * negative, a fixed coupling is used.
   */
  inline Energy2 lambdaQCD2() const;

  /**
   * Set the scale of the last generated emission.
   */
  inline void lastPT2(Energy2);
  //@}

protected:

  /** @name Functions relating to the DipoleState to which this belongs. */
  //@{
  /**
   * Fill the provided set with all pointers to CloneBase objects used
   * in this object.
   * @return a vector of pointers.
   */
  inline virtual void fillReferences(CloneSet &);

  /**
   * Rebind pointer to other CloneBase objects. Called after a number
   * of interconnected CloneBase objects have been cloned, so that
   * the cloned objects will refer to the cloned copies afterwards.
   *
   * @param trans a TranslationMap relating the original objects to
   * their respective clones.
   */
  inline virtual void rebind(const TranslationMap & trans);
  //@}

public:

  /** @name Functions used by the persistent I/O system. */
  //@{
  /**
   * Function used to write out object persistently.
   * @param os the persistent output stream written to.
   */
  void persistentOutput(PersistentOStream & os) const;

  /**
   * Function used to read in object persistently.
   * @param is the persistent input stream read from.
   * @param version the version number of the object when written.
   */
  void persistentInput(PersistentIStream & is, int version);
  //@}

  /**
   * The standard Init function used to initialize the interfaces.
   * Called exactly once for each class by the class description system
   * before the main function starts or
   * when this class is dynamically loaded.
   */
  static void Init();

protected:

  /**
   * Exception class used if weights larger that one is encountered.
   */
  struct WeightException: Exception {};

private:

  /**
   * The scale of the last generated emission.
   */
  Energy2 theLastPT2;

  /**
   * The constant \f$\alpha_S\f$ or the constant to be divided by the
   * log of the scale.
   */
  double theAlpha0;

  /**
   * The \f$\lambda{QCD}^2\f$ used in the running \f$\alpha_S\f$. If
   * negative, a fixed coupling is used.
   */
  Energy2 theLambdaQCD2;

private:

  /**
   * The static object used to initialize the description of this class.
   * Indicates that this is an abstract class with persistent data.
   */
  static AbstractClassDescription<Emitter> initEmitter;

  /**
   * The assignment operator is private and must never be called.
   * In fact, it should not even be implemented.
   */
  Emitter & operator=(const Emitter &);

};

}

#include "ThePEG/Utilities/ClassTraits.h"

namespace ThePEG {

/** @cond TRAITSPECIALIZATIONS */

/** This template specialization informs ThePEG about the
 *  base classes of Emitter. */
template <>
struct BaseClassTrait<Ariadne::Emitter,1> {
  /** Typedef of the first base class of Emitter. */
  typedef Ariadne::CascadeBase NthBase;
};

/** This template specialization informs ThePEG about the name of
 *  the Emitter class and the shared object where it is defined. */
template <>
struct ClassTraits<Ariadne::Emitter>
  : public ClassTraitsBase<Ariadne::Emitter> {
  /** Return a platform-independent class name */
  static string className() { return "Ariadne::Emitter"; }
  /** Return the name of the shared library be loaded to get
   *  access to the Emitter class and every other class it uses
   *  (except the base class). */
  static string library() { return "libArCascade.so"; }
};

/** @endcond */

}

#include "Emitter.icc"
#ifndef ThePEG_TEMPLATES_IN_CC_FILE
// #include "Emitter.tcc"
#endif

#endif /* ARIADNE_Emitter_H */
