// -*- C++ -*-
//
// This is the implementation of the non-inlined, non-templated member
// functions of the StringFragmentation class.
//

#include "StringFragmentation.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/Reference.h"
#include "ThePEG/PDT/RemnantDecayer.h"   
#include "ThePEG/EventRecord/Particle.h"
#include "ThePEG/EventRecord/Step.h"
#include "ThePEG/Handlers/EventHandler.h"


#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"

using namespace TheP8I;

StringFragmentation::StringFragmentation()
  :  pythia(),
#include "StringFragmentation-init.h"
      {}

StringFragmentation::~StringFragmentation() {}

void StringFragmentation::handle(EventHandler & eh, const tPVector & tagged,
	    const Hint & h) {
  if ( !pythia.created() ) pythia.init(*this, theAdditionalP8Settings);

  pythia.clearEvent();

  tPVector all = RemnantDecayer::decayRemnants(tagged, *newStep());
  vector<ColourSinglet> singlets;
  if ( theCollapser ) singlets =  theCollapser->collapse(all, newStep());
  else singlets = ColourSinglet::getSinglets(all.begin(), all.end());

  for ( int i = 0, N = singlets.size(); i < N; ++i ) {

    if ( singlets[i].nPieces() == 3 ) {
      // Simple junction.
      // Save place where we will store dummy particle.
      int nsave = pythia.event().size();
      pythia.event().append(22, -21, 0, 0, 0, 0, 0, 0, 0.0, 0.0, 0.0, 0.0);
      int npart = 0;
      for ( int ip = 1; ip <= 3; ++ip )
	for ( int j = 0, M = singlets[i].piece(ip).size(); j < M; ++j ) {
	  pythia.addParticle(singlets[i].piece(ip)[j], 23, nsave, 0);
	  ++npart;
	}
      pythia.event()[nsave].daughters(nsave + 1, nsave + npart);  
    }
    else if ( singlets[i].nPieces() == 5 ) {
      // Double, connected junction
      // Save place where we will store dummy beam particles.
      int nb1 = pythia.event().size();
      int nb2 = nb1 + 1;
      pythia.event().append(2212, -21, 0, 0, nb1 + 2, nb1 + 4, 0, 0,
			    0.0, 0.0, 0.0, 0.0);
      pythia.event().append(-2212, -21, 0, 0, nb2 + 4, nb2 + 6, 0, 0,
			    0.0, 0.0, 0.0, 0.0);

      // Find the string piece connecting the junctions, and the
      // other loose string pieces.
      int connector = 0;
      int q1 = 0;
      int q2 = 0;
      int aq1 = 0;
      int aq2 = 0;
      for ( int ip = 1; ip <= 5; ++ip ) {
	if ( singlets[i].sink(ip).first && singlets[i].source(ip).first )
	    connector = ip;
	else if ( singlets[i].source(ip).first ) {
	  if ( q1 ) q2 = ip;
	  else q1 = ip;
	}
	else if ( singlets[i].sink(ip).first ) {
	  if ( aq1 ) aq2 = ip;
	  else aq1 = ip;
	}
      }
      if ( !connector || !q1 || !q2 || !aq1 || ! aq2 )
	Throw<StringFragError>()
	  << name() << " found complicated junction string. Although Pytha8 can "
	  << "hadronize junction strings, this one was too complicated."
	  << Exception::runerror;
      
      // Insert the partons of the loose triplet ends.
      int start = pythia.event().size();
      for ( int j = 0, M = singlets[i].piece(q1).size(); j < M; ++j )
	pythia.addParticle(singlets[i].piece(q1)[j], 23, nb1, 0);
      for ( int j = 0, M = singlets[i].piece(q2).size(); j < M; ++j )
	pythia.addParticle(singlets[i].piece(q2)[j], 23, nb1, 0);
      // Insert dummy triplet incoming parton with correct colour code.
      int col1 = singlets[i].piece(connector).empty()? pythia.event().nextColTag():
	pythia.addColourLine(singlets[i].piece(connector).front()->colourLine());
      int dum1 = pythia.event().size();
      pythia.event().append(2, -21, nb1, 0, 0, 0, col1, 0, 0.0, 0.0, 0.0, 0.0, 0.0);
      pythia.event()[nb1].daughters(start, start + singlets[i].piece(q1).size() +
				    singlets[i].piece(q2).size());

      // Insert the partons of the loose anti-triplet ends.
      start = pythia.event().size();
      for ( int j = 0, M = singlets[i].piece(aq1).size(); j < M; ++j )
	pythia.addParticle(singlets[i].piece(aq1)[j], 23, nb2, 0);
      for ( int j = 0, M = singlets[i].piece(aq2).size(); j < M; ++j )
	pythia.addParticle(singlets[i].piece(aq2)[j], 23, nb2, 0);
      // Insert dummy anti-triplet incoming parton with correct colour code.
      int col2 = singlets[i].piece(connector).empty()? col1:
	pythia.addColourLine(singlets[i].piece(connector).back()->antiColourLine());
      int dum2 = pythia.event().size();
      pythia.event().append(-2, -21, nb2, 0, 0, 0, 0, col2, 0.0, 0.0, 0.0, 0.0, 0.0);
      pythia.event()[nb2].daughters(start, start + singlets[i].piece(aq1).size() +
				    singlets[i].piece(aq2).size());

      // Add the partons from the connecting string piece.
      for ( int j = 0, M = singlets[i].piece(connector).size(); j < M; ++j )
	pythia.addParticle(singlets[i].piece(connector)[j], 23, dum1, dum2);
    }
    else if ( singlets[i].nPieces() > 1 ) {
      // We don't know how to handle other junctions yet.
      Throw<StringFragError>()
	<< name() << " found complicated junction string. Although Pytha8 can "
	<< "hadronize junction strings, that interface is not ready yet."
	<< Exception::runerror;
    } else {
      // Normal string
      for ( int j = 0, M = singlets[i].partons().size(); j < M; ++j )
	pythia.addParticle(singlets[i].partons()[j], 23, 0, 0);
    }

  }

  for ( int i = 0, N = all.size(); i < N; ++i )
    if ( !all[i]->coloured() )
      pythia.addParticle(all[i], 23, 0, 0);

  int oldsize = pythia.event().size();

  if ( !pythia.go() ) {
    pythia.event().list();
    cerr << *eh.currentEvent();
    Throw<StringFragError>()
      << "Pythia8 failed to hadronize partonic state:\n"
      << Exception::runerror;
  }
  for ( int i = oldsize, N = pythia.event().size(); i < N; ++i )
    pythia.getParticle(i);

  for ( int i = oldsize, N = pythia.event().size(); i < N; ++i ) {
    PPtr p = pythia.getParticle(i);
    if ( !p ) continue;
    Pythia8::Particle & pyp = pythia.event()[i];
    if ( pyp.mother1() <= 0 ) continue;
    if ( pyp.mother1() == pyp.mother2() ) {
      newStep()->setCopy(pythia.getParticle(pyp.mother1()), p);
    }
    else if ( pyp.mother2() <= 0 ) {
      newStep()->addDecayProduct(pythia.getParticle(pyp.mother1()), p);
    }
    else if ( pyp.mother2() > pyp.mother1() ) {
      tPVector parents;
      for ( int j = pyp.mother1(); j <= pyp.mother2(); ++j ) {
	tPPtr par = pythia.getParticle(j);
	if ( par ) parents.push_back(par);
      }
      newStep()->addDecayProduct(parents.begin(), parents.end(), p);
    }
  }
}

IBPtr StringFragmentation::clone() const {
  return new_ptr(*this);
}

IBPtr StringFragmentation::fullclone() const {
  return new_ptr(*this);
}


void StringFragmentation::doinitrun() {
  HadronizationHandler::doinitrun();
  theAdditionalP8Settings.push_back("ProcessLevel:all = off");
  theAdditionalP8Settings.push_back("HadronLevel:Decay = off");
  pythia.init(*this, theAdditionalP8Settings);

}

void StringFragmentation::persistentOutput(PersistentOStream & os) const {
  os
#include "StringFragmentation-output.h"
    << theCollapser;
}

void StringFragmentation::persistentInput(PersistentIStream & is, int) {
  is
#include "StringFragmentation-input.h"
    >> theCollapser;
}

ClassDescription<StringFragmentation> StringFragmentation::initStringFragmentation;
// Definition of the static class description member.

void StringFragmentation::Init() {

#include "StringFragmentation-interfaces.h"


  static Reference<StringFragmentation,ClusterCollapser> interfaceCollapser
    ("Collapser",
     "A ThePEG::ClusterCollapser object used to collapse colour singlet "
     "clusters which are too small to fragment. If no object is given the "
     "MinistringFragmentetion of Pythia8 is used instead.",
     &StringFragmentation::theCollapser, true, false, true, true, false);


}

